// *****************************************************************************
//  Game   version:  1.0   Ankur Sheel  date: 04/30/2008
//  ----------------------------------------------------------------------------
//  
//  ----------------------------------------------------------------------------
//  Copyright (C) 2008 - All Rights Reserved
// *****************************************************************************
// 
// *****************************************************************************
#include "stdafx.h"
#include "Game.h"
#include "Sprite.hxx"
#include "MPongView.h"
#include "Timer.hxx"
#include "ProcessManager.hxx"
#include "MainWindow.hxx"
#include "ResourceManager.hxx"
#include "EntityManager.hxx"
#include "MessageDispatchManager.hxx"
#include "BaseControl.hxx"
#include "PongGameElement.h"
#include "Paddle.h"
#include "GraphicUtils.hxx"
#include "Wall.h"
#include "Ball.h"
#include "Score.h"

using namespace Graphics;
using namespace Base;
using namespace GameBase;
using namespace Utilities;

// *****************************************************************************
cGame::cGame(const cString strName)
: cBaseApp(strName)
, m_pScore(NULL)
, m_ppGameElements(NULL)
{
}

// *****************************************************************************
cGame::~cGame()
{
	VCleanup();
}

// *****************************************************************************
void cGame::VOnInitialization(const HINSTANCE & hInstance, const int nCmdShow,
							  const cString & strOptionsFile)
{
	cBaseApp::VOnInitialization(hInstance, nCmdShow, strOptionsFile);

	m_iDisplayHeight = static_cast<int>(m_pHumanView->m_pAppWindowControl->VGetHeight());
	m_iDisplayWidth = static_cast<int>(m_pHumanView->m_pAppWindowControl->VGetWidth());

	cWindowControlDef HUDDef;
	HUDDef.wType = cWindowControlDef::WT_STANDARD;
	HUDDef.vSize = m_pHumanView->m_pAppWindowControl->VGetSize();
	HUDDef.vPosition = cVector2(0.0f, 0.0f);
	IBaseControl * pHUDScreen = IBaseControl::CreateWindowControl(HUDDef);
	m_pHumanView->m_pAppWindowControl->VAddChildControl(shared_ptr<IBaseControl>(pHUDScreen));

	cPongGameElement::SetGame(this);
	m_ppGameElements = DEBUG_NEW cPongGameElement*[PGE_TOTAL];
	for (int i=0; i<PGE_TOTAL; i++)
	{
		m_ppGameElements[i] = NULL;
	}

	m_vScreenTopLeftPos = IGraphicUtils::GetInstance()->ScreenToWorldSpace(cVector2(0,0), m_pHumanView->GetCamera());
	m_vScreenBottomRightPos = IGraphicUtils::GetInstance()->ScreenToWorldSpace(cVector2(static_cast<float>(m_iDisplayWidth), static_cast<float>(m_iDisplayHeight)),
		m_pHumanView->GetCamera());

	cGameElementDef paddleDef;
	paddleDef.strModelName= "cube";
	paddleDef.vPosition= cVector3(m_vScreenTopLeftPos.x, 0.0f, 0.0f);
	paddleDef.vScale = cVector3(0.5f, 1.5f, 0.5f);
	m_ppGameElements[PGE_PADDLE_LEFT] = DEBUG_NEW cPaddle();
	m_ppGameElements[PGE_PADDLE_LEFT]->VInitialize(paddleDef);

	paddleDef.vPosition= cVector3(m_vScreenBottomRightPos.x, 0.0f, 0.0f);
	m_ppGameElements[PGE_PADDLE_RIGHT] = DEBUG_NEW cPaddle();
	m_ppGameElements[PGE_PADDLE_RIGHT]->VInitialize(paddleDef);

	cGameElementDef wallDef;
	wallDef.strModelName = "cube";
	wallDef.vPosition= cVector3(0, m_vScreenTopLeftPos.y, 0.0f);
	wallDef.vScale = cVector3(abs(m_vScreenTopLeftPos.x), 0.5f, 0.5f);
	m_ppGameElements[PGE_WALL_UP] = DEBUG_NEW cWall();
	m_ppGameElements[PGE_WALL_UP]->VInitialize(wallDef);

	wallDef.vPosition= cVector3(0, m_vScreenBottomRightPos.y, 0.0f);
	m_ppGameElements[PGE_WALL_DOWN] = DEBUG_NEW cWall();
	m_ppGameElements[PGE_WALL_DOWN]->VInitialize(wallDef);

	cGameElementDef ballDef;
	ballDef.strModelName = "sphere";
	ballDef.vScale = cVector3(0.5f, 0.5f, 0.5f);
	m_ppGameElements[PGE_BALL] = DEBUG_NEW cBall();
	m_ppGameElements[PGE_BALL]->VInitialize(ballDef);

	m_pScore = DEBUG_NEW cScore[2]();
	
	m_pScore[0].Init(cVector2(0.0f, 0.0f));
	pHUDScreen->VAddChildControl(m_pScore[0].GetLabel());
	pHUDScreen->VMoveToFront(m_pScore[0].GetLabel().get());

	m_pScore[1].Init(cVector2((float)m_iDisplayWidth, 0.0f));
	pHUDScreen->VAddChildControl(m_pScore[1].GetLabel());
	pHUDScreen->VMoveToFront(m_pScore[1].GetLabel().get());
	
	//m_pHumanView->SetCursorVisible(false);
}

void cGame::VCreateHumanView()
{
	m_pHumanView = DEBUG_NEW cMPongView();
}

// *****************************************************************************
cString cGame::VGetGameTitle() const
{
	return VGetName();
}

// *****************************************************************************
void cGame::VOnUpdate()
{
	cBaseApp::VOnUpdate();
	for(int i=0; i<PGE_TOTAL; i++)
	{
		if(m_ppGameElements[i])
		{
			m_ppGameElements[i]->OnUpdate(m_pGameTimer->VGetDeltaTime());
		}
	}
	m_pHumanView->VOnUpdate(m_pGameTimer->VGetRunningTicks(), m_pGameTimer->VGetDeltaTime());
	HandlePaddleAI(m_pGameTimer->VGetDeltaTime());
}

// *****************************************************************************
void cGame::Restart()
{
	Sleep(100);
	m_ppGameElements[PGE_PADDLE_LEFT]->OnRestart();
	m_ppGameElements[PGE_PADDLE_RIGHT]->OnRestart();
	m_ppGameElements[PGE_BALL]->OnRestart();
}

// *****************************************************************************
void cGame::VRoundOver(const bool bPlayer1Won)
{
	if (bPlayer1Won)
	{
		m_pScore[1].IncrementScore();
	}
	else
	{
		m_pScore[0].IncrementScore();
	}
	Restart();
}

// *****************************************************************************
void cGame::VCleanup()
{
	if(m_ppGameElements)
	{
		for (int i=0; i<PGE_TOTAL; i++)
		{
			SafeDelete(&m_ppGameElements[i]);
		}
		SafeDeleteArray(&m_ppGameElements);
	}

	SafeDeleteArray(&m_pScore);

	cBaseApp::VCleanup();
}

// *****************************************************************************
void cGame::HandlePaddleAI( const float fElapsedTime )
{
	cVector3 vBallPos = m_ppGameElements[PGE_BALL]->GetPosition();
	// if the ball is in the players half, there is no need to do anything
	if (vBallPos.x < 0)
	{
		return;
	}
	cVector3 vPaddlePos = m_ppGameElements[PGE_PADDLE_RIGHT]->GetPosition();

	if (vPaddlePos.y - vBallPos.y < 1)
	{
		cPaddle * pPaddle = m_ppGameElements[PGE_PADDLE_RIGHT]->CastToPaddle();
		if(pPaddle)
		{
			pPaddle->MoveUp(fElapsedTime);
		}
		return;
	}
	else if (vBallPos.y - vPaddlePos.y < 1)
	{
		cPaddle * pPaddle = m_ppGameElements[PGE_PADDLE_RIGHT]->CastToPaddle();
		if(pPaddle)
		{
			pPaddle->MoveDown(fElapsedTime);
		}
		return;
	}
}

void cGame::MoveLeftPaddle(bool bMoveDown)
{
	cPaddle * pPaddle = m_ppGameElements[PGE_PADDLE_LEFT]->CastToPaddle();
	if(pPaddle)
	{
		if (bMoveDown)
		{
			pPaddle->MoveDown(m_pGameTimer->VGetDeltaTime());
		}
		else
		{
			pPaddle->MoveUp(m_pGameTimer->VGetDeltaTime());
		}
	}
}

// *****************************************************************************
bool cGame::VOnHandleMessage(const AI::Telegram & telegram)
{
	return false;
}

// *****************************************************************************
cPongGameElement ** const cGame::VGetGameElements() const
{
	return m_ppGameElements; 
}

// *****************************************************************************
cVector3 cGame::VGetScreenTopLeftPos() const
{
	return m_vScreenTopLeftPos;
}

// *****************************************************************************
cVector3 cGame::VGetScreenBottomRightPos() const
{
	return m_vScreenBottomRightPos;
}